class Packet
  def self.parse(data)
    data = data.each_char.to_a
    bytes = []
    data.each_slice(2) do |characters|
      bytes << characters.join("").to_i(16)
    end
    return if bytes == [0,0,0]

    case bytes.first & 3
    when 0
      CountPacket.new(bytes)
    when 1
      TemperaturePacket.new(bytes)
    when 2
      PowerPacket.new(bytes)
    else
      Packet.new(bytes)
    end
  end

  def initialize(data)
    @data = data
  end

  attr_reader :data

  def inspect
    "#{name}:#{@data.inspect}"
  end

  def to_s
    inspect
  end

  def name
    "Unknown"
  end
end

class TemperaturePacket < Packet
  def name
    "T"
  end

  def temperature
    (119 - (20/26.0)*data[1]).floor
  end
  def to_s
    "<TemperaturePacket F=#{temperature}>"
  end
end

class CountPacket < Packet
  def name
    "C"
  end

  def to_s
    "<CountPacket Count=#{(data[1]<<8)+data[0]}>"
  end
end

class PowerPacket < Packet
  def name
    "P"
  end

  def to_s
    "<PowerPacket Data=#{data.inspect}>"
  end
end
